//---------------------------------------------------------------------------
#include <vcl.h>
#include <stdio.h>
#include <windows.h>
#pragma hdrstop
#define MAXPATH 260
#include "GetDriveSpace.h"
#pragma package(smart_init)


// Wrapper for the WIN API call
typedef BOOL (WINAPI *P_GDFSE)(LPCTSTR, PULARGE_INTEGER,
                                 PULARGE_INTEGER, PULARGE_INTEGER);
unsigned __int64 i64FreeBytesToCaller;
unsigned __int64 i64TotalBytes;
unsigned __int64 i64FreeBytes;

DWORD dwSectPerClust,
      dwBytesPerSect,
      dwFreeClusters,
      dwTotalClusters;

//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.

static inline void ValidCtrCheck(TGetDriveSpace *)
{
    new TGetDriveSpace(NULL);
}
//---------------------------------------------------------------------------
__fastcall TGetDriveSpace::TGetDriveSpace(TComponent* Owner)
    : TComponent(Owner)
{
}
//---------------------------------------------------------------------------
namespace Getdrivespace
{
    void __fastcall PACKAGE Register()
    {
        TComponentClass classes[1] = {__classid(TGetDriveSpace)};
        RegisterComponents("PWA", classes, 0);
    }
}
//---------------------------------------------------------------------------
void gds(char *Drive,  unsigned __int64 &iFree, unsigned __int64 &iTot)
   {
      BOOL  fResult;
      char  szDrive[MAXPATH];
      char  pszDrive[MAXPATH];

      P_GDFSE pGetDiskFreeSpaceEx = NULL;
      /*  This section modified from a Microsoft white paper
         Command line parsing.
         If the drive is a drive letter and not a UNC path, append a
         trailing backslash to the drive letter and colon.  This is
         required on Windows 95 and 98.
      */
      strcpy(pszDrive,Drive);
      memset (szDrive,'\0',MAXPATH);
      memset (pszDrive,'\0',MAXPATH);
      strcpy(pszDrive,Drive);
      if (pszDrive[1] == ':')
      {
         szDrive[0] = pszDrive[0];
         szDrive[1] = ':';
         szDrive[2] = '\\';
         szDrive[3] = '\0';
         strcpy(pszDrive,szDrive);
      }

      /* Use GetDiskFreeSpaceEx if available; otherwise, use
         GetDiskFreeSpace.
         Note: Since GetDiskFreeSpaceEx is not in Windows 95 Retail, we
         dynamically link to it and only call it if it is present.  We
         don't need to call LoadLibrary on KERNEL32.DLL because it is
         already loaded into every Win32 process's address space.
      */
      pGetDiskFreeSpaceEx = (P_GDFSE)GetProcAddress (
                               GetModuleHandle ("kernel32.dll"),
                                                "GetDiskFreeSpaceExA");
      if (pGetDiskFreeSpaceEx)
      {
         fResult = pGetDiskFreeSpaceEx (pszDrive,
                                 (PULARGE_INTEGER)&i64FreeBytesToCaller,
                                 (PULARGE_INTEGER)&i64TotalBytes,
                                 (PULARGE_INTEGER)&i64FreeBytes);
         if (fResult)
         {
           iFree = i64FreeBytesToCaller;
           iTot = i64TotalBytes;
         }
         else
         {
            iFree = 0;
            iTot=0;
         }
      }
      else
      {
         /* If GetFreeDiskSpaceEX did not exist, call the
            older GetFreeDiskSpace */
         fResult = GetDiskFreeSpace (pszDrive,
                                     &dwSectPerClust,
                                     &dwBytesPerSect,
                                     &dwFreeClusters,
                                     &dwTotalClusters);
         if (fResult)
         {
            /* force 64-bit math to accomidate modern drive sizes*/
            i64TotalBytes = (unsigned __int64)dwTotalClusters * dwSectPerClust *
                              dwBytesPerSect;
            i64FreeBytes = (unsigned __int64)dwFreeClusters * dwSectPerClust *
                              dwBytesPerSect;
            iFree = i64FreeBytes;
            iTot = i64TotalBytes;
         }
            else
         {
            iFree = 0;
            iTot=0;
         }

      }

   }

//---------------------------------------------------------------------------

  void __fastcall TGetDriveSpace::SetDrive (AnsiString ADrive)
  {
    /* Component fires when the drive letter is changed.  Usefull
       at design time.*/
    FDrive = ADrive;
    if (ADrive.Length() >1)
    Execute();
  }
  void __fastcall TGetDriveSpace::SetType(TstrType AType)
  {
    FstrType = AType;
  }
  void __fastcall TGetDriveSpace::SetAnsiFree (AnsiString AstrFree)
  {
    /* create a neatly formatted string of the free space based
        on the strType enumerated property. */
    char tmp[MAXPATH];
    sprintf(tmp,"%I64u",(unsigned __int64) FnumFree);
    FstrFree = tmp;
    if(FnumFree == 0)
        FstrFree = "Not a valid Drive";
    else
    {
        if (FstrType ==0)
            FstrFree = FstrFree.FormatFloat("#,##0;;Zero",FnumFree)  + " Bytes";
        if (FstrType ==1)
            FstrFree = FstrFree.FormatFloat("#,##0;;Zero",FnumFree /1000)+ " KBytes";
        if (FstrType ==2)
            FstrFree = FstrFree.FormatFloat("#,##0;;Zero",FnumFree /1000000)+ " MBytes";
        if (FstrType ==3)
            FstrFree = FstrFree.FormatFloat("#,##0.0;;Zero",FnumFree /1000000000) + " GBytes";
    }
  }
  void __fastcall TGetDriveSpace::SetAnsiTot (AnsiString AstrTot)

  {
    /* create a neatly formatted string of the total space based
        on the strType enumerated property. */
    char tmp[MAXPATH];
    sprintf(tmp,"%I64u",(unsigned __int64)FnumTot);
    FstrTot = tmp;
    if(FnumFree == 0)
        FstrFree = "Not a valid Drive";
    else
    {
        if (FstrType ==0)
            FstrTot = FstrTot.FormatFloat("#,##0;;Zero",FnumTot) + " Bytes";
        if (FstrType == 1)
            FstrTot = FstrTot.FormatFloat("#,##0;;Zero",FnumTot /1000) + " KBytes";
        if (FstrType == 2)
            FstrTot = FstrTot.FormatFloat("#,##0;;Zero",FnumTot /1000000) + " MBytes";
        if (FstrType == 3)
            FstrTot = FstrTot.FormatFloat("#,##0.0;;Zero",FnumTot /1000000000) + " GBytes";
    }
  }
  void __fastcall TGetDriveSpace::SetFree (double AnumFree)
  {
    FnumFree = AnumFree;
  }
  void __fastcall TGetDriveSpace::SetTot (double AnumTot)
  {
    FnumTot = AnumTot;
  }
  void __fastcall TGetDriveSpace::Execute()
  {
    unsigned __int64 iFree;
    unsigned __int64 iTot;
    gds(FDrive.c_str(),iFree,iTot);
    SetFree(iFree);
    SetTot(iTot);
    SetAnsiTot("");
    SetAnsiFree("");

  }

